@@ -66,11 +66,9 @@ in this Software without prior written authorization from The Open Group.
 #include "events.h"
 #include "resize.h"
 #include "parse.h"
-#include "gram.h"
 #include "util.h"
 #include "screen.h"
 #include "icons.h"
-#include "version.h"
 
 #ifdef HAVE_XRANDR
 #include <X11/extensions/Xrandr.h>
@@ -86,7 +84,7 @@ XEvent Event;                   /* the current event */
 static TwmWindow *Tmp_win;      /* the current twm window */
 
 /** Used in HandleEnterNotify to remove border highlight from a window
- * that has not recieved a LeaveNotify event because of a pointer grab
+ * that has not received a LeaveNotify event because of a pointer grab
  */
 static TwmWindow *UnHighLight_win = NULL;
 
@@ -344,7 +342,7 @@ DispatchEvent(void)
  * handle X events
  */
 void
-HandleEvents(void)
+HandleEvents(XtAppContext appContext)
 {
     while (TRUE) {
         if (enter_flag && !QLength(dpy)) {
@@ -380,9 +378,8 @@ void
 HandleColormapNotify(void)
 {
     XColormapEvent *cevent = (XColormapEvent *) &Event;
-    ColormapWindow *cwin, **cwins;
+    ColormapWindow *cwin;
     TwmColormap *cmap;
-    int lost, won, n, number_cwins;
     XPointer context_data;
 
     if (XFindContext(dpy, cevent->window, ColormapContext, &context_data) == 0)
@@ -392,7 +389,11 @@ HandleColormapNotify(void)
 
     cmap = cwin->colormap;
 
+#if defined(__cplusplus) || defined(c_plusplus)
+    if (cevent->c_new) {
+#else
     if (cevent->new) {
+#endif
         if (XFindContext(dpy, cevent->colormap, ColormapContext,
                          &context_data) == XCNOENT)
             cwin->colormap = CreateTwmColormap(cevent->colormap);
@@ -430,6 +431,9 @@ HandleColormapNotify(void)
         }
 
         if (cevent->serial >= Scr->cmapInfo.first_req) {
+            ColormapWindow **cwins;
+            int lost, won, n, number_cwins;
+
             number_cwins = Scr->cmapInfo.cmaps->number_cwins;
 
             /*
@@ -537,7 +541,7 @@ HandleVisibilityNotify(void)
         return;
 
     /*
-     * when Saber complains about retreiving an <int> from an <unsigned int>
+     * when Saber complains about retrieving an <int> from an <unsigned int>
      * just type "touch vevent->state" and "cont"
      */
     cmap = cwin->colormap;
@@ -629,7 +633,7 @@ HandleKeyPress(void)
                     for (Tmp_win = Scr->TwmRoot.next; Tmp_win != NULL;
                          Tmp_win = Tmp_win->next) {
                         if (!strncmp
-                            (key->win_name, Tmp_win->class.res_name,
+                            (key->win_name, Tmp_win->xclass.res_name,
                              (size_t) len)) {
                             matched = TRUE;
                             ExecuteFunction(key->func, key->action,
@@ -644,7 +648,7 @@ HandleKeyPress(void)
                     for (Tmp_win = Scr->TwmRoot.next; Tmp_win != NULL;
                          Tmp_win = Tmp_win->next) {
                         if (!strncmp
-                            (key->win_name, Tmp_win->class.res_class,
+                            (key->win_name, Tmp_win->xclass.res_class,
                              (size_t) len)) {
                             matched = TRUE;
                             ExecuteFunction(key->func, key->action,
@@ -698,13 +702,14 @@ free_window_names(TwmWindow *tmp, Bool nukefull, Bool nukename, Bool nukeicon)
 void
 free_cwins(TwmWindow *tmp)
 {
-    int i;
-    TwmColormap *cmap;
-
     if (tmp->cmaps.number_cwins) {
+        int i;
+
         for (i = 0; i < tmp->cmaps.number_cwins; i++) {
+
             if (--tmp->cmaps.cwins[i]->refcnt == 0) {
-                cmap = tmp->cmaps.cwins[i]->colormap;
+                TwmColormap *cmap = tmp->cmaps.cwins[i]->colormap;
+
                 if (--cmap->refcnt == 0) {
                     XDeleteContext(dpy, cmap->c, ColormapContext);
                     free(cmap);
@@ -714,6 +719,7 @@ free_cwins(TwmWindow *tmp)
             }
         }
         free(tmp->cmaps.cwins);
+
         if (tmp->cmaps.number_cwins > 1) {
             free(tmp->cmaps.scoreboard);
             tmp->cmaps.scoreboard = NULL;
@@ -729,9 +735,11 @@ void
 HandlePropertyNotify(void)
 {
     char *name = NULL;
-    unsigned long valuemask;    /* mask for create windows */
     XSetWindowAttributes attributes;    /* attributes for create windows */
     Pixmap pm;
+    int dummy = 0;
+    unsigned udummy = 0;
+    Window wdummy = None;
 
     /* watch for standard colormap changes */
     if (Event.xproperty.window == Scr->Root) {
@@ -818,9 +826,9 @@ HandlePropertyNotify(void)
                // to a default icon */
             int icon_x = 0, icon_y = 0;
 
-            XGetGeometry(dpy, Tmp_win->icon_w, &JunkRoot,
+            XGetGeometry(dpy, Tmp_win->icon_w, &wdummy,
                          &icon_x, &icon_y,
-                         &JunkWidth, &JunkHeight, &JunkBW, &JunkDepth);
+                         &udummy, &udummy, &udummy, &udummy);
             XSelectInput(dpy, Tmp_win->icon_w, None);
             XDeleteContext(dpy, Tmp_win->icon_w, TwmContext);
             XDeleteContext(dpy, Tmp_win->icon_w, ScreenContext);
@@ -838,9 +846,9 @@ HandlePropertyNotify(void)
                  * Try to find out where it is; if we succeed, move the new
                  * window to where the old one is.
                  */
-                if (XGetGeometry(dpy, Tmp_win->icon_w, &JunkRoot, &icon_x,
-                                 &icon_y, &JunkWidth, &JunkHeight, &JunkBW,
-                                 &JunkDepth)) {
+                if (XGetGeometry(dpy, Tmp_win->icon_w, &wdummy, &icon_x,
+                                 &icon_y, &udummy, &udummy, &udummy,
+                                 &udummy)) {
                     /*
                      * Move the new icon window to where the old one was.
                      */
@@ -894,11 +902,13 @@ HandlePropertyNotify(void)
 
         if (Tmp_win->icon_w && !Tmp_win->forced && Tmp_win->wmhints &&
             (Tmp_win->wmhints->flags & IconPixmapHint)) {
-            if (!XGetGeometry(dpy, Tmp_win->wmhints->icon_pixmap, &JunkRoot,
-                              &JunkX, &JunkY,
+            unsigned long valuemask;    /* mask for create windows */
+
+            if (!XGetGeometry(dpy, Tmp_win->wmhints->icon_pixmap, &wdummy,
+                              &dummy, &dummy,
                               (unsigned int *) &Tmp_win->icon_width,
-                              (unsigned int *) &Tmp_win->icon_height, &JunkBW,
-                              &JunkDepth)) {
+                              (unsigned int *) &Tmp_win->icon_height, &udummy,
+                              &udummy)) {
                 return;
             }
 
@@ -1011,15 +1021,19 @@ RedoIconName(void)
 void
 HandleClientMessage(void)
 {
+    int dummy = 0;
+    unsigned udummy = 0;
+    Window wdummy = None;
+
     if (Event.xclient.message_type == _XA_WM_CHANGE_STATE) {
         if (Tmp_win != NULL) {
             if (Event.xclient.data.l[0] == IconicState && !Tmp_win->icon) {
                 XEvent button;
 
-                XQueryPointer(dpy, Scr->Root, &JunkRoot, &JunkChild,
+                XQueryPointer(dpy, Scr->Root, &wdummy, &wdummy,
                               &(button.xmotion.x_root),
                               &(button.xmotion.y_root),
-                              &JunkX, &JunkY, &JunkMask);
+                              &dummy, &dummy, &udummy);
 
                 ExecuteFunction(F_ICONIFY, NULLSTR, Event.xany.window,
                                 Tmp_win, &button, FRAME, FALSE);
@@ -1089,12 +1103,12 @@ HandleExpose(void)
         else if (Tmp_win->titlebuttons) {
             int i;
             Window w = Event.xany.window;
-            register TBWindow *tbw;
+            TBWindow *tbw;
             int nb = Scr->TBInfo.nleft + Scr->TBInfo.nright;
 
             for (i = 0, tbw = Tmp_win->titlebuttons; i < nb; i++, tbw++) {
                 if (w == tbw->window) {
-                    register TitleButton *tb = tbw->info;
+                    TitleButton *tb = tbw->info;
 
                     FB(Tmp_win->title.fore, Tmp_win->title.back);
                     XCopyPlane(dpy, tb->bitmap, w, Scr->NormalGC,
@@ -1167,7 +1181,6 @@ remove_window_from_ring(TwmWindow *tmp)
 void
 HandleDestroyNotify(void)
 {
-    int i;
 
     /*
      * Warning, this is also called by HandleUnmapNotify; if it ever needs to
@@ -1198,7 +1211,10 @@ HandleDestroyNotify(void)
             XDeleteContext(dpy, Tmp_win->hilite_w, TwmContext);
             XDeleteContext(dpy, Tmp_win->hilite_w, ScreenContext);
         }
+
         if (Tmp_win->titlebuttons) {
+            int i;
+
             for (i = 0; i < nb; i++) {
                 XDeleteContext(dpy, Tmp_win->titlebuttons[i].window,
                                TwmContext);
@@ -1218,8 +1234,8 @@ HandleDestroyNotify(void)
      *     2.  name
      *     3.  icon_name
      *     4.  wmhints
-     *     5.  class.res_name
-     *     6.  class.res_class
+     *     5.  xclass.res_name
+     *     6.  xclass.res_class
      *     7.  list
      *     8.  iconmgrp
      *     9.  cwins
@@ -1244,10 +1260,12 @@ HandleDestroyNotify(void)
     free_window_names(Tmp_win, True, True, True);       /* 1, 2, 3 */
     if (Tmp_win->wmhints)       /* 4 */
         XFree(Tmp_win->wmhints);
-    if (Tmp_win->class.res_name && Tmp_win->class.res_name != NoName)   /* 5 */
-        XFree(Tmp_win->class.res_name);
-    if (Tmp_win->class.res_class && Tmp_win->class.res_class != NoName) /* 6 */
-        XFree(Tmp_win->class.res_class);
+    if (Tmp_win->xclass.res_name &&
+        Tmp_win->xclass.res_name != NoName)     /* 5 */
+        XFree(Tmp_win->xclass.res_name);
+    if (Tmp_win->xclass.res_class &&
+        Tmp_win->xclass.res_class != NoName)    /* 6 */
+        XFree(Tmp_win->xclass.res_class);
     free_cwins(Tmp_win);        /* 9 */
     if (Tmp_win->titlebuttons)  /* 10 */
         free(Tmp_win->titlebuttons);
@@ -1279,7 +1297,6 @@ void
 HandleMapRequest(void)
 {
     XPointer context_data;
-    int zoom_save;
 
     Event.xany.window = Event.xmaprequest.window;
     if (XFindContext(dpy, Event.xany.window, TwmContext, &context_data) == 0)
@@ -1309,6 +1326,7 @@ HandleMapRequest(void)
     if ((!Tmp_win->icon) &&
         Tmp_win->wmhints && (Tmp_win->wmhints->flags & StateHint)) {
         int state;
+        int zoom_save;
         Window icon;
 
         /* use WM_STATE if enabled */
@@ -1459,12 +1477,14 @@ void
 HandleMotionNotify(void)
 {
     XPointer context_data;
+    unsigned udummy = 0;
+    Window wdummy = None;
 
     if (ResizeWindow != (Window) 0) {
         XQueryPointer(dpy, Event.xany.window,
-                      &(Event.xmotion.root), &JunkChild,
+                      &(Event.xmotion.root), &wdummy,
                       &(Event.xmotion.x_root), &(Event.xmotion.y_root),
-                      &(Event.xmotion.x), &(Event.xmotion.y), &JunkMask);
+                      &(Event.xmotion.x), &(Event.xmotion.y), &udummy);
 
         /* Set WindowMoved appropriately so that f.deltastop will
            work with resize as well as move. */
@@ -1484,7 +1504,6 @@ HandleMotionNotify(void)
 void
 HandleButtonRelease(void)
 {
-    int xl, xr, yt, yb, w, h;
     unsigned mask;
     XPointer context_data;
 
@@ -1496,6 +1515,9 @@ HandleButtonRelease(void)
         }
 
     if (DragWindow != None) {
+        int xl, yt;
+        int w, h;
+
         MoveOutline(Scr->Root, 0, 0, 0, 0, 0, 0);
 
         if (XFindContext(dpy, DragWindow, TwmContext, &context_data) == 0)
@@ -1529,8 +1551,8 @@ HandleButtonRelease(void)
         }
 
         if (Scr->DontMoveOff && MoveFunction != F_FORCEMOVE) {
-            xr = xl + w;
-            yb = yt + h;
+            int xr = xl + w;
+            int yb = yt + h;
 
             if (xl < 0)
                 xl = 0;
@@ -1603,7 +1625,7 @@ HandleButtonRelease(void)
             Context = C_NO_CONTEXT;
             ButtonWindow = NULL;
 
-            /* if we are not executing a defered command, then take down the
+            /* if we are not executing a deferred command, then take down the
              * menu
              */
             if (RootFunction == 0) {
@@ -1688,6 +1710,8 @@ HandleButtonPress(void)
 {
     unsigned int modifier;
     Cursor cur;
+    int dummy = 0;
+    Window wdummy = None;
 
     /* too much code relies on this assumption */
     if (Event.xbutton.button > MAX_BUTTONS)
@@ -1739,8 +1763,8 @@ HandleButtonPress(void)
 
     /* check the title bar buttons */
     if (Tmp_win && Tmp_win->title_height && Tmp_win->titlebuttons) {
-        register int i;
-        register TBWindow *tbw;
+        int i;
+        TBWindow *tbw;
         int nb = Scr->TBInfo.nleft + Scr->TBInfo.nright;
 
         for (i = 0, tbw = Tmp_win->titlebuttons; i < nb; i++, tbw++) {
@@ -1773,13 +1797,17 @@ HandleButtonPress(void)
         if (Tmp_win->list && RootFunction != 0 &&
             (Event.xany.window == Tmp_win->list->w ||
              Event.xany.window == Tmp_win->list->icon)) {
+            int x = 0;
+            int y = 0;
+
             Tmp_win = Tmp_win->list->iconmgr->twm_win;
+
             XTranslateCoordinates(dpy, Event.xany.window, Tmp_win->w,
                                   Event.xbutton.x, Event.xbutton.y,
-                                  &JunkX, &JunkY, &JunkChild);
+                                  &x, &y, &wdummy);
 
-            Event.xbutton.x = JunkX;
-            Event.xbutton.y = JunkY - Tmp_win->title_height;
+            Event.xbutton.x = x;
+            Event.xbutton.y = y - Tmp_win->title_height;
             Event.xany.window = Tmp_win->w;
 
             Context = C_WINDOW;
@@ -1835,11 +1863,13 @@ HandleButtonPress(void)
              * inside of a client that was getting button press events.
              */
             XPointer context_data;
+            int x = 0;
+            int y = 0;
 
             XTranslateCoordinates(dpy, Scr->Root, Scr->Root,
                                   Event.xbutton.x,
                                   Event.xbutton.y,
-                                  &JunkX, &JunkY, &Event.xany.window);
+                                  &dummy, &dummy, &Event.xany.window);
 
             if (Event.xany.window == 0 ||
                 (XFindContext(dpy, Event.xany.window, TwmContext,
@@ -1854,10 +1884,10 @@ HandleButtonPress(void)
 
             XTranslateCoordinates(dpy, Scr->Root, Event.xany.window,
                                   Event.xbutton.x,
-                                  Event.xbutton.y, &JunkX, &JunkY, &JunkChild);
+                                  Event.xbutton.y, &x, &y, &wdummy);
 
-            Event.xbutton.x = JunkX;
-            Event.xbutton.y = JunkY;
+            Event.xbutton.x = x;
+            Event.xbutton.y = y;
 
             Context = C_WINDOW;
         }
@@ -2218,7 +2248,6 @@ void
 HandleConfigureRequest(void)
 {
     XWindowChanges xwc;
-    unsigned long xwcm;
     int x, y, width, height, bw;
     int gravx, gravy;
     XConfigureRequestEvent *cre = &Event.xconfigurerequest;
@@ -2258,6 +2287,8 @@ HandleConfigureRequest(void)
      * to configuration requests for windows which have never been mapped.
      */
     if (!Tmp_win || Tmp_win->icon_w == cre->window) {
+        unsigned long xwcm;
+
         xwcm = cre->value_mask &
             (CWX | CWY | CWWidth | CWHeight | CWBorderWidth);
         xwc.x = cre->x;
@@ -2420,10 +2451,12 @@ ScreenInfo *
 FindScreenInfo(Window w)
 {
     XWindowAttributes attr;
-    int scrnum;
 
     attr.screen = NULL;
+
     if (XGetWindowAttributes(dpy, w, &attr)) {
+        int scrnum;
+
         for (scrnum = 0; scrnum < NumScreens; scrnum++) {
             if (ScreenList[scrnum] != NULL &&
                 (ScreenOfDisplay(dpy, ScreenList[scrnum]->screen) ==
@@ -2498,7 +2531,7 @@ InstallWindowColormaps(int type, TwmWindow *tmp)
 
     state = CM_INSTALLED;
 
-    for (i = n = 0; i < number_cwins; i++) {
+    for (i = 0; i < number_cwins; i++) {
         cwin = cwins[i];
         cmap = cwin->colormap;
         cmap->state |= CM_INSTALLABLE;
