@@ -49,7 +49,6 @@ in this Software without prior written authorization from The Open Group.
 /**    OR PERFORMANCE OF THIS SOFTWARE.                                     **/
 /*****************************************************************************/
 
-
 /**********************************************************************
  *
  * GC related externs
@@ -58,9 +57,9 @@ in this Software without prior written authorization from The Open Group.
  *
  **********************************************************************/
 
-#ifndef _GC_
-#define _GC_
+#ifndef GC_H
+#define GC_H
 
-extern void CreateGCs( void );
+extern void CreateGCs(void);
 
-#endif /* _GC_ */
+#endif /* GC_H */
