@@ -59,7 +59,7 @@ in this Software without prior written authorization from The Open Group.
 
 #include "twm.h"
 #include "util.h"
-#include "gram.h"
+#include "parse.h"
 #include "screen.h"
 #include <X11/Xos.h>
 #include <X11/Xatom.h>
@@ -96,7 +96,7 @@ MoveOutline(Window root, int x, int y, int width, int height, int bw, int th)
     int xl, xr, yt, yb, xinnerl, xinnerr, yinnert, yinnerb;
     int xthird, ythird;
     XSegment outline[18];
-    register XSegment *r;
+    XSegment *r;
 
     if (x == lastx && y == lasty && width == lastWidth && height == lastHeight
         && lastBW == bw && th == lastTH)
@@ -209,6 +209,8 @@ Zoom(Window wf, Window wt)
     long dx, dy, dw, dh;
     long z;
     int j;
+    unsigned udummy = 0;
+    Window wdummy = None;
 
     if (!Scr->DoZoom || Scr->ZoomCount < 1)
         return;
@@ -216,8 +218,8 @@ Zoom(Window wf, Window wt)
     if (wf == None || wt == None)
         return;
 
-    XGetGeometry(dpy, wf, &JunkRoot, &fx, &fy, &fw, &fh, &JunkBW, &JunkDepth);
-    XGetGeometry(dpy, wt, &JunkRoot, &tx, &ty, &tw, &th, &JunkBW, &JunkDepth);
+    XGetGeometry(dpy, wf, &wdummy, &fx, &fy, &fw, &fh, &udummy, &udummy);
+    XGetGeometry(dpy, wt, &wdummy, &tx, &ty, &tw, &th, &udummy, &udummy);
 
     dx = ((long) (tx - fx));    /* going from -> to */
     dy = ((long) (ty - fy));    /* going from -> to */
@@ -257,13 +259,11 @@ ExpandFilename(const char *name)
     if (name[0] != '~')
         return strdup(name);
 
-    newname = malloc((size_t) HomeLen + strlen(name) + 2);
+    newname = (char *) malloc((size_t) HomeLen + strlen(name) + 2);
     if (!newname) {
-        fprintf(stderr,
-                "%s:  unable to allocate %ld bytes to expand filename %s/%s\n",
-                ProgramName,
-                (unsigned long) HomeLen + (unsigned long) strlen(name) + 2,
-                Home, &name[1]);
+        twmWarning("unable to allocate %lu bytes to expand filename %s/%s",
+                   (unsigned long) HomeLen + (unsigned long) strlen(name) + 2,
+                   Home, &name[1]);
     }
     else {
         (void) sprintf(newname, "%s/%s", Home, &name[1]);
@@ -280,10 +280,14 @@ ExpandFilename(const char *name)
 void
 GetUnknownIcon(const char *name)
 {
+    int dummy = 0;
+    unsigned udummy = 0;
+    Window wdummy = None;
+
     if ((Scr->UnknownPm = GetBitmap(name)) != None) {
-        XGetGeometry(dpy, Scr->UnknownPm, &JunkRoot, &JunkX, &JunkY,
+        XGetGeometry(dpy, Scr->UnknownPm, &wdummy, &dummy, &dummy,
                      (unsigned int *) &Scr->UnknownWidth,
-                     (unsigned int *) &Scr->UnknownHeight, &JunkBW, &JunkDepth);
+                     (unsigned int *) &Scr->UnknownHeight, &udummy, &udummy);
     }
 }
 
@@ -331,8 +335,7 @@ FindBitmap(const char *name, unsigned *widthp, unsigned *heightp)
             if (XmuCompareISOLatin1(pmtab[i].name, name) == 0)
                 return (*pmtab[i].proc) (widthp, heightp);
         }
-        fprintf(stderr, "%s:  no such built-in bitmap \"%s\"\n",
-                ProgramName, name);
+        twmWarning("no such built-in bitmap \"%s\"", name);
         return None;
     }
 
@@ -355,11 +358,11 @@ FindBitmap(const char *name, unsigned *widthp, unsigned *heightp)
         /*
          * Attempt to find icon in old IconDirectory (now obsolete)
          */
-        bigname = malloc(strlen(name) + strlen(Scr->IconDirectory) + 2);
+        bigname = (char *)
+            malloc(strlen(name) + strlen(Scr->IconDirectory) + 2);
         if (!bigname) {
-            fprintf(stderr,
-                    "%s:  unable to allocate memory for \"%s/%s\"\n",
-                    ProgramName, Scr->IconDirectory, name);
+            twmWarning("unable to allocate memory for \"%s/%s\"",
+                       Scr->IconDirectory, name);
             return None;
         }
         (void) sprintf(bigname, "%s/%s", Scr->IconDirectory, name);
@@ -370,8 +373,7 @@ FindBitmap(const char *name, unsigned *widthp, unsigned *heightp)
     }
     free(bigname);
     if (pm == None) {
-        fprintf(stderr, "%s:  unable to find bitmap \"%s\"\n",
-                ProgramName, name);
+        twmWarning("unable to find bitmap \"%s\"", name);
     }
 
     return pm;
@@ -380,7 +382,9 @@ FindBitmap(const char *name, unsigned *widthp, unsigned *heightp)
 Pixmap
 GetBitmap(const char *name)
 {
-    return FindBitmap(name, &JunkWidth, &JunkHeight);
+    unsigned udummy = 0;
+
+    return FindBitmap(name, &udummy, &udummy);
 }
 
 void
@@ -396,10 +400,10 @@ InsertRGBColormap(Atom a, XStandardColormap *maps, int nmaps, Bool replace)
     }
 
     if (!sc) {                  /* no existing, allocate new */
-        sc = malloc(sizeof(StdCmap));
+        sc = (StdCmap *) malloc(sizeof(StdCmap));
         if (!sc) {
-            fprintf(stderr, "%s:  unable to allocate %ld bytes for StdCmap\n",
-                    ProgramName, (unsigned long) sizeof(StdCmap));
+            twmWarning("unable to allocate %lu bytes for StdCmap",
+                       (unsigned long) sizeof(StdCmap));
             return;
         }
         replace = False;
@@ -483,10 +487,8 @@ GetColor(int kind, Pixel *what, const char *name)
     Status stat = 0;
     Colormap cmap = Scr->TwmRoot.cmaps.cwins[0]->colormap->c;
 
-#ifndef TOM
     if (!Scr->FirstTime)
         return;
-#endif
 
     if (Scr->Monochrome != kind)
         return;
@@ -501,8 +503,7 @@ GetColor(int kind, Pixel *what, const char *name)
         if (name[0] != '#')
             stat = XParseColor(dpy, cmap, name, &color);
         if (!stat) {
-            fprintf(stderr, "%s:  invalid color name \"%s\"\n",
-                    ProgramName, name);
+            twmWarning("invalid color name \"%s\"", name);
             return;
         }
 
@@ -545,8 +546,7 @@ GetColor(int kind, Pixel *what, const char *name)
                             stdcmap->blue_mult));
         }
         else {
-            fprintf(stderr, "%s:  unable to allocate color \"%s\"\n",
-                    ProgramName, name);
+            twmWarning("unable to allocate color \"%s\"", name);
             return;
         }
     }
@@ -560,16 +560,14 @@ GetColorValue(int kind, XColor *what, const char *name)
     XColor junkcolor;
     Colormap cmap = Scr->TwmRoot.cmaps.cwins[0]->colormap->c;
 
-#ifndef TOM
     if (!Scr->FirstTime)
         return;
-#endif
 
     if (Scr->Monochrome != kind)
         return;
 
     if (!XLookupColor(dpy, cmap, name, what, &junkcolor)) {
-        fprintf(stderr, "%s:  invalid color name \"%s\"\n", ProgramName, name);
+        twmWarning("invalid color name \"%s\"", name);
     }
     else {
         what->pixel = AllPlanes;
@@ -585,12 +583,13 @@ FindFontSet(MyFont *font, const char *fontname)
     XFontSetExtents *font_extents;
     XFontStruct **xfonts;
     char **font_names;
-    register int i;
-    int ascent;
-    int descent;
-    int fnum;
 
     if (use_fontset) {
+        int ascent;
+        int descent;
+        int fnum;
+        int i;
+
         if (font->fontset != NULL) {
             XFreeFontSet(dpy, font->fontset);
         }
@@ -601,9 +600,14 @@ FindFontSet(MyFont *font, const char *fontname)
                                             &def_string_return)) == NULL) {
             return False;
         }
-        for (i = 0; i < missing_charset_count_return; i++) {
-            printf("%s: warning: font for charset %s is lacking.\n",
-                   ProgramName, missing_charset_list_return[i]);
+        if (missing_charset_count_return) {
+            twmVerbose("%d fonts are missing from fontset",
+                       missing_charset_count_return);
+            for (i = 0; i < missing_charset_count_return; i++) {
+                twmVerbose("font for charset %s is lacking.",
+                           missing_charset_list_return[i]);
+            }
+            XFreeStringList(missing_charset_list_return);
         }
 
         font_extents = XExtentsOfFontSet(font->fontset);
@@ -619,6 +623,9 @@ FindFontSet(MyFont *font, const char *fontname)
         font->y = ascent;
         font->ascent = ascent;
         font->descent = descent;
+        twmMessage("created fontset with %d fonts (%d missing) for \"%s\"",
+                   fnum, missing_charset_count_return,
+                   fontname ? fontname : "NULL");
         return True;
     }
 
@@ -648,10 +655,10 @@ FindFontSet(MyFont *font, const char *fontname)
 void
 GetFont(MyFont *font)
 {
-    const char *deffontname = "fixed";
 
     if (!FindFontSet(font, font->name)) {
         const char *what = "fonts";
+        const char *deffontname = "fixed";
 
         if (use_fontset) {
             what = "fontsets";
@@ -660,13 +667,30 @@ GetFont(MyFont *font)
             deffontname = Scr->DefaultFont.name;
         }
         if (!FindFontSet(font, deffontname)) {
-            fprintf(stderr, "%s:  unable to open %s \"%s\" or \"%s\"\n",
-                    ProgramName, what, font->name, deffontname);
-            exit(1);
+            twmError("unable to open %s \"%s\" or \"%s\"",
+                     what, font->name, deffontname);
         }
     }
 }
 
+void
+DestroyFont(MyFont *font)
+{
+    if (!font) {
+        return;
+    }
+
+    if (font->fontset) {
+        XFreeFontSet(dpy, font->fontset);
+        font->fontset = NULL;
+    }
+
+    if (font->font) {
+        XFreeFont(dpy, font->font);
+        font->font = NULL;
+    }
+}
+
 int
 MyFont_TextWidth(MyFont *font, const char *string, int len)
 {
@@ -729,23 +753,38 @@ I18N_FetchName(Display *dpy2, Window w, char **winname)
 {
     int status;
     XTextProperty text_prop;
-    char **list;
-    int num;
+    int rc = 0;
+
+    *winname = NULL;
 
     status = XGetWMName(dpy2, w, &text_prop);
-    if (!status || !text_prop.value || !text_prop.nitems) {
-        *winname = NULL;
-        return 0;
+    if (status && text_prop.value && text_prop.nitems) {
+        char **list = NULL;
+        int num;
+
+        status = XmbTextPropertyToTextList(dpy2, &text_prop, &list, &num);
+        if (status >= Success && num && list && *list) {
+            XFree(text_prop.value);
+            *winname = strdup(*list);
+            XFreeStringList(list);
+            rc = 1;
+        }
+        else {
+            char *value = NULL;
+
+            /*
+             * If the system's locale support is broken (e.g., missing useful
+             * parts), the preceding Xmb call may fail.
+             */
+            if (XFetchName(dpy2, w, &value) && value != NULL) {
+                *winname = strdup(value);
+                XFree(value);
+                rc = 1;
+            }
+        }
     }
-    status = XmbTextPropertyToTextList(dpy2, &text_prop, &list, &num);
-    if (status < Success || !num || !*list) {
-        *winname = NULL;
-        return 0;
-    }
-    XFree(text_prop.value);
-    *winname = (char *) strdup(*list);
-    XFreeStringList(list);
-    return 1;
+
+    return rc;
 }
 
 Status
@@ -779,11 +818,11 @@ SetFocus(TwmWindow *tmp_win, Time time)
 
 #ifdef TRACE
     if (tmp_win) {
-        printf("Focusing on window \"%s\"\n", tmp_win->full_name);
+        twmMessage("Focusing on window \"%s\"", tmp_win->full_name);
     }
     else {
-        printf("Unfocusing; Scr->Focus was \"%s\"\n",
-               Scr->Focus ? Scr->Focus->full_name : "(nil)");
+        twmMessage("Unfocusing; Scr->Focus was \"%s\"",
+                   Scr->Focus ? Scr->Focus->full_name : "(nil)");
     }
 #endif
 
@@ -900,11 +939,11 @@ CreateDotPixmap(unsigned *widthp, unsigned *heightp)
     if (!(h & 1))
         h--;
     *widthp = *heightp = (unsigned int) h;
-    if (Scr->tbpm.delete == None) {
+    if (Scr->tbpm.remove == None) {
         GC gc;
         Pixmap pix;
 
-        pix = Scr->tbpm.delete =
+        pix = Scr->tbpm.remove =
             XCreatePixmap(dpy, Scr->Root, (unsigned) h, (unsigned) h, 1);
         gc = XCreateGC(dpy, pix, 0L, NULL);
         XSetLineAttributes(dpy, gc, (unsigned) h, LineSolid, CapRound,
@@ -915,7 +954,7 @@ CreateDotPixmap(unsigned *widthp, unsigned *heightp)
         XDrawLine(dpy, pix, gc, h / 2, h / 2, h / 2, h / 2);
         XFreeGC(dpy, gc);
     }
-    return Scr->tbpm.delete;
+    return Scr->tbpm.remove;
 }
 
 #define questionmark_width 8
@@ -952,15 +991,6 @@ Pixmap
 CreateMenuIcon(int height, unsigned *widthp, unsigned *heightp)
 {
     int h, w;
-    int ih, iw;
-    int ix, iy;
-    int mh, mw;
-    int tw, th;
-    int lw, lh;
-    int lx, ly;
-    int lines, dly;
-    int off;
-    int bw;
 
     h = height;
     w = h * 7 / 8;
@@ -973,6 +1003,15 @@ CreateMenuIcon(int height, unsigned *widthp, unsigned *heightp)
     if (Scr->tbpm.menu == None) {
         Pixmap pix;
         GC gc;
+        int ih, iw;
+        int ix, iy;
+        int mh, mw;
+        int tw, th;
+        int lw, lh;
+        int lx, ly;
+        int lines, dly;
+        int off;
+        int bw;
 
         pix = Scr->tbpm.menu =
             XCreatePixmap(dpy, Scr->Root, (unsigned) w, (unsigned) h, 1);
